// AnimationFrame.java
//
// Informatics 45 Spring 2010
// Code Example: Ball Animation (Version 1)
//
// There's nothing here that we haven't seen before.  This class implements
// our animation window, in which the animation is shown, along with buttons
// that can be used to start and stop it.

package inf45.spring2010.examples.animation1.gui;

import inf45.spring2010.examples.animation1.model.Animation;
import inf45.spring2010.examples.animation1.model.AnimationListener;
import inf45.spring2010.examples.animation1.model.AnimationState;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


public class AnimationFrame extends JFrame implements AnimationListener
{
	private AnimationPanel animationPanel;
	
	private Animation animation;
	

	public AnimationFrame()
	{
		setTitle("Informatics 45 Spring 2010 - Ball Animation #1");
		setSize(450, 500);
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

		animation = new Animation(30.0);
		animation.addAnimationListener(this);
		
		buildUI();
	}
	
	
	private void buildUI()
	{
		GridBagLayout layout = new GridBagLayout();
		getContentPane().setLayout(layout);
		
		animationPanel = new AnimationPanel();
		getContentPane().add(animationPanel);
		layout.setConstraints(
			animationPanel,
			new GridBagConstraints(
				0, 0, 2, 1, 1.0, 1.0,
				GridBagConstraints.CENTER, GridBagConstraints.BOTH,
				new Insets(10, 10, 10, 10), 0, 0));
		
		JButton startButton = new JButton("Start");
		startButton.addActionListener(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					startAnimation();
				}
			});
		getContentPane().add(startButton);
		layout.setConstraints(
			startButton,
			new GridBagConstraints(
				0, 1, 1, 1, 1.0, 0.0,
				GridBagConstraints.EAST, GridBagConstraints.NONE,
				new Insets(5, 5, 5, 5), 0, 0));
		
		JButton stopButton = new JButton("Stop");
		stopButton.addActionListener(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					stopAnimation();
				}
			});
		getContentPane().add(stopButton);
		layout.setConstraints(
			stopButton,
			new GridBagConstraints(
				1, 1, 1, 1, 0.0, 0.0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE,
				new Insets(5, 5, 5, 5), 0, 0));
	}
	
	
	private void startAnimation()
	{
		animation.start();
	}
	
	
	private void stopAnimation()
	{
		animation.stop();
	}


	// These are the implementations of the methods from the AnimationListener
	// interface.  Since we expect them to be called from the animation thread,
	// but since we are not permitted to make UI-related changes on any thread
	// other than the GUI thread, we use the EventQueue.invokeLater() method
	// that we've used in the past, to ensure that any work we do is done on
	// the GUI thread.
	
	public void animationStarted()
	{
	}
	
	
	public void animationUpdated(final AnimationState newState)
	{
		EventQueue.invokeLater(
			new Runnable()
			{
				public void run()
				{
					animationPanel.updateState(newState);
					animationPanel.repaint();
				}
			});
	}
	
	
	public void animationStopped()
	{
		EventQueue.invokeLater(
			new Runnable()
			{
				public void run()
				{
					animationPanel.updateState(null);
					animationPanel.repaint();
				}
			});
	}
	
	
	// dispose() is called automatically when the window is closed.  In this
	// case, we'll stop the animation thread, so that the program can end.
	public void dispose()
	{
		stopAnimation();
		super.dispose();
	}
}
